@@ -291,9 +291,7 @@ class HMMAligner(torch.nn.Module):
 
         return poss_phns, log_transition_matrix, start_states, final_states
 
-    def use_lexicon(
-        self, words, interword_sils=True, sample_pron=False,
-    ):
+    def use_lexicon(self, words, interword_sils=True, sample_pron=False):
         """Do processing using the lexicon to return a sequence of the possible
         phonemes, the transition/pi probabilities, and the possible final
         states.
@@ -1042,7 +1040,9 @@ class HMMAligner(torch.nn.Module):
         batch_size = len(lens_abs)
         fb_max_length = torch.max(lens_abs)
 
-        flat_start_batch = torch.zeros(batch_size, fb_max_length).long()
+        flat_start_batch = torch.zeros(
+            batch_size, fb_max_length, device=phns.device
+        ).long()
         for i in range(batch_size):
             utter_phns = phns[i]
             utter_phns = utter_phns[: phn_lens_abs[i]]  # crop out zero padding
@@ -1091,7 +1091,9 @@ class HMMAligner(torch.nn.Module):
         batch_size = len(lens_abs)
         fb_max_length = torch.max(lens_abs)
 
-        viterbi_batch = torch.zeros(batch_size, fb_max_length).long()
+        viterbi_batch = torch.zeros(
+            batch_size, fb_max_length, device=lens_abs.device
+        ).long()
         for i in range(batch_size):
             viterbi_preds = self.align_dict[ids[i]]
             viterbi_preds = torch.nn.functional.pad(
@@ -1418,7 +1420,7 @@ def batch_log_matvecmul(A, b):
     >>> x
     tensor([[0.6931, 0.0000]])
     >>>
-    >>> # non-log domain equivalent without batching funcionality
+    >>> # non-log domain equivalent without batching functionality
     >>> A_ = torch.tensor([[1., 1.],
     ...                    [0., 1.]])
     >>> b_ = torch.tensor([1., 1.,])
