@@ -5,11 +5,13 @@ Authors
  * Mirco Ravanelli 2020
  * Nauman Dawalatabad 2020
  * Jianyuan Zhong 2020
+ * Sarthak Yadav 2022
 """
 
 import torch
 import logging
 import torch.nn as nn
+import torch.nn.functional as F
 
 logger = logging.getLogger(__name__)
 
@@ -105,7 +107,13 @@ class Pooling1d(nn.Module):
             raise ValueError("pool_type must be 'avg' or 'max'")
 
     def forward(self, x):
+        """Performs 1d pooling to the input tensor.
 
+        Arguments
+        ---------
+        x : torch.Tensor
+            It represents a tensor for a mini-batch.
+        """
         # Put the pooling axes as the last dimension for torch.nn.pool
         x = x.transpose(-1, self.pool_axis)
 
@@ -188,7 +196,13 @@ class Pooling2d(nn.Module):
             )
 
     def forward(self, x):
+        """Performs 2d pooling to the input tensor.
 
+        Arguments
+        ---------
+        x : torch.Tensor
+            It represents a tensor for a mini-batch.
+        """
         # Add extra two dimension at the last two, and then swap the pool_axis to them
         # Example: pool_axis=[1,2]
         # [a,b,c,d] => [a,b,c,d,1,1]
@@ -364,9 +378,150 @@ class AdaptivePool(nn.Module):
             self.pool = nn.AdaptiveAvgPool2d(output_size)
 
     def forward(self, x):
+        """Performs adpative pooling to the input tensor.
 
+        Arguments
+        ---------
+        x : torch.Tensor
+            It represents a tensor for a mini-batch.
+        """
         if x.ndim == 3:
             return self.pool(x.permute(0, 2, 1)).permute(0, 2, 1)
 
         if x.ndim == 4:
             return self.pool(x.permute(0, 3, 1, 2)).permute(0, 2, 3, 1)
+
+
+class GaussianLowpassPooling(nn.Module):
+    """
+    This class implements a learnable Gaussian lowpass pooling from
+
+    Neil Zeghidour, Olivier Teboul, F{\'e}lix de Chaumont Quitry & Marco Tagliasacchi, "LEAF: A LEARNABLE FRONTEND
+    FOR AUDIO CLASSIFICATION", in Proc. of ICLR 2021 (https://arxiv.org/abs/2101.08596)
+
+    Arguments
+    ---------
+    in_channels : int
+        The number of input channels.
+    kernel_size: int
+        Kernel size of the gaussian lowpass filters.
+    stride : int
+        Stride factor of the convolutional filters. When the stride factor > 1,
+        a decimation in time is performed.
+    padding : str
+        (same, valid). If "valid", no padding is performed.
+        If "same" and stride is 1, output shape is the same as the input shape.
+    padding_mode : str
+        This flag specifies the type of padding. See torch.nn documentation
+        for more information.
+    bias : bool
+        If True, the additive bias b is adopted.
+    skip_transpose : bool
+        If False, uses batch x time x channel convention of speechbrain.
+        If True, uses batch x channel x time convention.
+
+    Example
+    -------
+    >>> inp_tensor = torch.rand([10, 8000, 40])
+    >>> low_pass_pooling = GaussianLowpassPooling(
+    ...     40, kernel_size=401, stride=160,
+    ... )
+    >>> # parameters corresponding to a window of 25 ms and stride 10 ms at 16000 kHz
+    >>> out_tensor = low_pass_pooling(inp_tensor)
+    >>> out_tensor.shape
+    torch.Size([10, 50, 40])
+    """
+
+    def __init__(
+        self,
+        in_channels,
+        kernel_size,
+        stride=1,
+        initialization_constant=0.4,
+        padding="same",
+        padding_mode="constant",
+        bias=True,
+        skip_transpose=False,
+    ):
+        super(GaussianLowpassPooling, self).__init__()
+        self.kernel_size = kernel_size
+        self.stride = stride
+        self.padding = padding
+        self.padding_mode = padding_mode
+        self.in_channels = in_channels
+        self.skip_transpose = skip_transpose
+        self.weights = nn.Parameter(
+            torch.ones((1, 1, in_channels, 1)) * initialization_constant
+        )
+
+        if bias:
+            self._bias = torch.nn.Parameter(torch.ones(in_channels,))
+        else:
+            self._bias = None
+
+    def _get_impulse_responses(self, sigma):
+        filter_size = self.kernel_size
+        sigma = torch.clamp(sigma, min=(2.0 / filter_size), max=0.5)
+        t = torch.arange(0, filter_size, dtype=sigma.dtype, device=sigma.device)
+        t = torch.reshape(t, (1, filter_size, 1, 1))
+        numerator = t - 0.5 * (filter_size - 1)
+        denominator = sigma * 0.5 * (filter_size - 1)
+        return torch.exp(-0.5 * (numerator / denominator) ** 2)
+
+    def forward(self, x):
+        """Performs GaussianLowpass Pooling.
+
+        Arguments
+        ---------
+        x : torch.Tensor
+            3D tensor in input [batch,time,channels].
+        """
+        if not self.skip_transpose:
+            x = x.transpose(1, -1)
+
+        kernel = self._get_impulse_responses(self.weights)
+        kernel = kernel.reshape(-1, self.kernel_size, self.in_channels)
+        kernel = kernel.permute(2, 0, 1)
+
+        if self.padding == "same":
+            x = self._manage_padding(x, self.kernel_size)
+        elif self.padding == "valid":
+            pass
+        else:
+            raise ValueError(
+                "Padding must be 'same' or 'valid'. Got " + self.padding
+            )
+        outputs = F.conv1d(
+            x,
+            kernel,
+            bias=self._bias,
+            stride=self.stride,
+            padding=0,
+            groups=self.in_channels,
+        )
+        if not self.skip_transpose:
+            outputs = outputs.transpose(1, -1)
+        return outputs
+
+    def _manage_padding(self, x, kernel_size):
+        # this is the logic that gives correct shape that complies
+        # with the original implementation at https://github.com/google-research/leaf-audio
+
+        def get_padding_value(kernel_size):
+            """Get number of elements to pad."""
+            kernel_sizes = (kernel_size,)
+            from functools import reduce
+            from operator import __add__
+
+            conv_padding = reduce(
+                __add__,
+                [
+                    (k // 2 + (k - 2 * (k // 2)) - 1, k // 2)
+                    for k in kernel_sizes[::-1]
+                ],
+            )
+            return conv_padding
+
+        pad_value = get_padding_value(kernel_size)
+        x = F.pad(x, pad_value, mode=self.padding_mode, value=0)
+        return x
